"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var enumerable_1 = require("./enumerable");
var exceptions_1 = require("./exceptions");
var utils_1 = require("./utils");
var ReadOnlyCollection = /** @class */ (function () {
    function ReadOnlyCollection(list) {
        utils_1.throws.ThrowIfNull('list', list);
        this.list = list;
    }
    Object.defineProperty(ReadOnlyCollection.prototype, "Length", {
        get: function () {
            return this.list.Length;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(ReadOnlyCollection.prototype, "IsReadOnly", {
        get: function () {
            return true;
        },
        enumerable: true,
        configurable: true
    });
    ReadOnlyCollection.prototype.Set = function (index, item) {
        throw new exceptions_1.InvalidOperationException('不可向只读列表中写入数据');
    };
    ReadOnlyCollection.prototype.Get = function (index) {
        return this.list.Get(index);
    };
    ReadOnlyCollection.prototype.Add = function (item) {
        throw new exceptions_1.InvalidOperationException('不可向只读列表中写入数据');
    };
    ReadOnlyCollection.prototype.Clear = function () {
        throw new exceptions_1.InvalidOperationException('不可清空只读列表');
    };
    ReadOnlyCollection.prototype.CopyTo = function (array, arrayIndex) {
        this.list.CopyTo(array, arrayIndex);
    };
    ReadOnlyCollection.prototype.IndexOf = function (item) {
        return this.list.IndexOf(item);
    };
    ReadOnlyCollection.prototype.Insert = function (index, item) {
        throw new exceptions_1.InvalidOperationException('不可向只读列表中写入数据');
    };
    ReadOnlyCollection.prototype.Remove = function (item) {
        throw new exceptions_1.InvalidOperationException('不可从只读列表中删除数据');
    };
    ReadOnlyCollection.prototype.RemoveAt = function (index) {
        throw new exceptions_1.InvalidOperationException('不可从只读列表中删除数据');
    };
    ReadOnlyCollection.prototype[Symbol.iterator] = function () {
        return this.list[Symbol.iterator]();
    };
    ReadOnlyCollection.prototype.All = function (predicate) {
        return enumerable_1.Enumerable.All(this, predicate);
    };
    ReadOnlyCollection.prototype.Any = function (predicate) {
        return enumerable_1.Enumerable.Any(this, predicate);
    };
    ReadOnlyCollection.prototype.AsEnumerable = function () {
        return enumerable_1.Enumerable.AsEnumerable(this);
    };
    ReadOnlyCollection.prototype.Average = function (selector) {
        return enumerable_1.Enumerable.Average(this, selector);
    };
    ReadOnlyCollection.prototype.Concat = function (second) {
        return enumerable_1.Enumerable.Concat(this, second);
    };
    ReadOnlyCollection.prototype.Contains = function (value, comparer) {
        return enumerable_1.Enumerable.Contains(this, value, comparer);
    };
    ReadOnlyCollection.prototype.Count = function (predicate) {
        return enumerable_1.Enumerable.Count(this, predicate);
    };
    ReadOnlyCollection.prototype.DefaultIfEmpty = function (defaultValue) {
        return enumerable_1.Enumerable.DefaultIfEmpty(this, defaultValue);
    };
    ReadOnlyCollection.prototype.Distinct = function (comparer) {
        return enumerable_1.Enumerable.Distinct(this, comparer);
    };
    ReadOnlyCollection.prototype.ElementAt = function (index) {
        return enumerable_1.Enumerable.ElementAt(this, index);
    };
    ReadOnlyCollection.prototype.ElementAtOrDefault = function (defaultValue, index) {
        return enumerable_1.Enumerable.ElementAtOrDefault(this, defaultValue, index);
    };
    ReadOnlyCollection.prototype.Except = function (second, comparer) {
        return enumerable_1.Enumerable.Except(this, second, comparer);
    };
    ReadOnlyCollection.prototype.First = function (predicate) {
        return enumerable_1.Enumerable.First(this, predicate);
    };
    ReadOnlyCollection.prototype.FirstOrDefault = function (defaultValue, predicate) {
        return enumerable_1.Enumerable.FirstOrDefault(this, defaultValue, predicate);
    };
    ReadOnlyCollection.prototype.GroupBy = function (keySelector, elementSelector, comparer) {
        return enumerable_1.Enumerable.GroupBy(this, keySelector, elementSelector, comparer);
    };
    ReadOnlyCollection.prototype.GroupJoin = function (inner, outerKeySelector, innerKeySelector, resultSelector, comparer) {
        return enumerable_1.Enumerable.GroupJoin(this, inner, outerKeySelector, innerKeySelector, resultSelector, comparer);
    };
    ReadOnlyCollection.prototype.Intersect = function (second, comparer) {
        return enumerable_1.Enumerable.Intersect(this, second, comparer);
    };
    ReadOnlyCollection.prototype.Join = function (inner, outerKeySelector, innerKeySelector, resultSelector, comparer) {
        return enumerable_1.Enumerable.Join(this, inner, outerKeySelector, innerKeySelector, resultSelector, comparer);
    };
    ReadOnlyCollection.prototype.Last = function (predicate) {
        return enumerable_1.Enumerable.Last(this, predicate);
    };
    ReadOnlyCollection.prototype.LastOrDefault = function (defaultValue, predicate) {
        return enumerable_1.Enumerable.LastOrDefault(this, defaultValue, predicate);
    };
    ReadOnlyCollection.prototype.Max = function (selector) {
        return enumerable_1.Enumerable.Max(this, selector);
    };
    ReadOnlyCollection.prototype.Min = function (selector) {
        return enumerable_1.Enumerable.Min(this, selector);
    };
    ReadOnlyCollection.prototype.Reverse = function () {
        return enumerable_1.Enumerable.Reverse(this);
    };
    ReadOnlyCollection.prototype.Select = function (selector) {
        return enumerable_1.Enumerable.Select(this, selector);
    };
    ReadOnlyCollection.prototype.SelectMany = function (collectionSelector, resultSelector) {
        return enumerable_1.Enumerable.SelectMany(this, collectionSelector, resultSelector);
    };
    ReadOnlyCollection.prototype.SequenceEqual = function (second, comparer) {
        return enumerable_1.Enumerable.SequenceEqual(this, second, comparer);
    };
    ReadOnlyCollection.prototype.Single = function (predicate) {
        return enumerable_1.Enumerable.Single(this, predicate);
    };
    ReadOnlyCollection.prototype.SingleOrDefault = function (defaultValue, predicate) {
        return enumerable_1.Enumerable.SingleOrDefault(this, defaultValue, predicate);
    };
    ReadOnlyCollection.prototype.Skip = function (count) {
        return enumerable_1.Enumerable.Skip(this, count);
    };
    ReadOnlyCollection.prototype.SkipWhile = function (predicate) {
        return enumerable_1.Enumerable.SkipWhile(this, predicate);
    };
    ReadOnlyCollection.prototype.Sum = function (selector) {
        return enumerable_1.Enumerable.Sum(this, selector);
    };
    ReadOnlyCollection.prototype.Take = function (count) {
        return enumerable_1.Enumerable.Take(this, count);
    };
    ReadOnlyCollection.prototype.TakeWhile = function (predicate) {
        return enumerable_1.Enumerable.TakeWhile(this, predicate);
    };
    ReadOnlyCollection.prototype.ToArray = function () {
        return enumerable_1.Enumerable.ToArray(this);
    };
    ReadOnlyCollection.prototype.ToDictionary = function (keySelector, elementSelector, comparer) {
        return enumerable_1.Enumerable.ToDictionary(this, keySelector, elementSelector, comparer);
    };
    ReadOnlyCollection.prototype.ToList = function () {
        return enumerable_1.Enumerable.ToList(this);
    };
    ReadOnlyCollection.prototype.Union = function (second, comparer) {
        return enumerable_1.Enumerable.Union(this, second, comparer);
    };
    ReadOnlyCollection.prototype.Where = function (predicate) {
        return enumerable_1.Enumerable.Where(this, predicate);
    };
    return ReadOnlyCollection;
}());
exports.ReadOnlyCollection = ReadOnlyCollection;
//# sourceMappingURL=read-only-collection.js.map